import ipih

def start() -> None:
    from MobileHelperService.service import MobileHelperService, checker
    MobileHelperService(checker=checker).start(True)

if __name__ == '__main__':
    start()



import ipih

from enum import Enum
from typing import Callable, Any
from collections import defaultdict

from pih import A, IClosable

from MobileHelperService.api import (
    Flags,
    mio_command,
    MobileOutput,
    MobileSession,
    format_given_name,
    MobileHelperUserSettiongsHolder,
)
from pih.consts import ARGUMENT_PREFIX
from MobileHelperService.const import (
    SD,
    ADMIN_ALIAS,
    COMMAND_KEYWORDS,
    InterruptionTypes,
)
from pih.collections import WhatsAppMessage
from pih.tools import ne, n, j, nn, ParameterList
from pih.collections.service import ServiceDescription
from pih.consts.errors import OperationCanceled, OperationExit

ANSWER: dict[str, list[str]] = defaultdict(list)


class Client:
    class CT:
        S_UP = A.CT_AD.UserProperies

    @staticmethod
    def start(
        host: str,
        as_admin: bool = False,
        as_standalone: bool = False,
        version: str | None = None,
        pih_version: str | None = None,
        show_output: bool = True,
    ) -> bool:
        service_description: ServiceDescription = A.D.fill_data_from_source(
            ServiceDescription(), SD
        )  # type: ignore
        service_description.host = host or service_description.host
        service_description.parameters = (
            j((ARGUMENT_PREFIX, ADMIN_ALIAS)) if as_admin else None
        )
        service_description.pih_version = pih_version or service_description.pih_version
        service_description.use_standalone = as_standalone
        service_description.version = version or service_description.version
        return A.SRV_A.start(
            service_role_or_description=service_description,
            check_if_started=False,
            show_output=show_output,
        )  # type: ignore

    @staticmethod
    def create_output(recipient: str | Enum) -> MobileOutput:
        recipient = A.D.get(recipient)
        session: MobileSession = MobileSession(recipient, A.D.get(Flags.SILENCE))  # type: ignore
        recipient_as_whatsapp_group: bool = recipient.endswith(A.CT_ME_WH.GROUP_SUFFIX)  # type: ignore
        output: MobileOutput = MobileOutput(session)
        session.output = output
        if not recipient_as_whatsapp_group:
            output.user.get_formatted_given_name = lambda: format_given_name(
                session, output  # type: ignore
            )  # type: ignore
            session.say_hello(recipient)  # type: ignore
        return output

    @staticmethod
    def waiting_for_result(
        command: str,
        recipient: str | Enum,
        chat_id: str | Enum | None = None,
        flags: int | None = None,
        args: tuple[Any] | None = None,
    ) -> Any | None:
        recipient = A.D.get(recipient)
        chat_id = A.D.get(chat_id)
        return_result_key: str = A.D.uuid()
        A.A_MIO.send_command_to(
            command, recipient, chat_id, flags, return_result_key, args
        )

        class DH:
            exception: BaseException | None = None

        def internal_handler(pl: ParameterList, listener: IClosable) -> None:
            event, parameters = A.D_Ex_E.with_parameters(pl)
            if event == A.CT_E.RESULT_WAS_RETURNED:
                if parameters[0] == return_result_key:
                    if parameters[2] == A.D.get(InterruptionTypes.CANCEL):
                        DH.exception = OperationCanceled()
                    if parameters[2] == A.D.get(InterruptionTypes.EXIT):
                        DH.exception = OperationExit()
                    ANSWER[j((recipient, chat_id), "-")].append(parameters[1])
                    listener.close()

        A.E.on_event(internal_handler)
        if nn(DH.exception):
            raise DH.exception  # type: ignore
        return ANSWER[j((recipient, chat_id), "-")][-1]

    @staticmethod
    def ask(
        title: str,
        recipient: str | Enum,
        chat_id: str | Enum | None = None,
        flags: int | None = None,
    ) -> Any | None:
        return Client.waiting_for_result(
            mio_command(COMMAND_KEYWORDS.ASK), recipient, chat_id, flags, (title,)
        )

    @staticmethod
    def ask_yes_no(
        title: str,
        recipient: str | Enum,
        chat_id: str | Enum | None = None,
        flags: int | None = None,
    ) -> bool | None:
        return Client.waiting_for_result(
            mio_command(COMMAND_KEYWORDS.ASK_YES_NO),
            recipient,
            chat_id,
            flags,
            (title,),
        )

    @staticmethod
    def waiting_for_answer_from(
        recipient: str,
        handler: Callable[[str, Callable[[], None]], None] | None = None,
    ) -> str | None:
        def internal_handler(message: str, close_handler: Callable[[], None]) -> None:
            ANSWER[recipient].append(message)
            if n(handler):
                close_handler()
            else:
                handler(message, close_handler)  # type: ignore

        Client.waiting_for_input_from(recipient, internal_handler)
        return ANSWER[recipient][-1]

    @staticmethod
    def waiting_for_input_from(
        recipient: str,
        handler: Callable[[str, Callable[[], None]], None] | None = None,
    ) -> None:
        def internal_handler(pl: ParameterList, listener: IClosable) -> None:
            message: WhatsAppMessage | None = A.D_Ex_E.whatsapp_message(pl)
            if ne(message) and A.D_F.telephone_number(
                message.sender  # type: ignore
            ) == A.D_F.telephone_number(recipient):
                if n(handler):
                    listener.close()
                else:
                    handler(message.message, listener.close)  # type: ignore

        A.E.on_event(internal_handler)

    class SETTINGS:
        class USER(MobileHelperUserSettiongsHolder):
            pass



from MobileHelperService.api import (
    Flags,
    MobileInput,
    MobileOutput,
    MobileSession,
    MobileUserInput,
    MobileMarkInput,
    get_wappi_status,
    InternalInterrupt,
    MobileHelper as Api,
    AddressedInterruption,
)
from pih.consts.errors import NotFound
from MobileHelperService.const import *
from pih.console_api import LINE
from pih import A, PIH, Stdin, PIHThread
from pih.tools import (
    j,
    b,
    n,
    i,
    e,
    nn,
    nl,
    js,
    ne,
    lw,
    one,
    if_else,  
    ParameterList,
    BitMask as BM,
)
from pih.collections import WhatsAppMessage, User
from MobileHelperService.client import Client as MIO
from pih.collections.service import ServiceDescription, SubscribtionResult


from collections import defaultdict
from typing import Callable, Any

SC = A.CT_SC

ISOLATED: bool = False


def sender_is_cli(value: str) -> bool:
    return value in A.D.map(
        lambda item: item[1],
        A.D.filter(
            lambda item: item[0].lower().endswith("cli"),
            A.D.to_list(A.CT_ME_WH.GROUP, None),
        ),
    )


class MobileHelperService:

    is_admin: bool = False

    @staticmethod
    def as_admin() -> bool:
        return MobileHelperService.is_admin or not A.D.contains(A.SYS.host(), SD.host)

    @staticmethod
    def count() -> int:
        return A.SE.named_arg(COUNT_ALIAS)  # type: ignore

    mobile_helper_client_map: dict[str, Api] = {}

    def __init__(
        self,
        max_client_count: int | None = None,
        checker: Callable[[str], bool] | None = None,
    ):
        self.max_client_count: int | None = max_client_count or DEFAULT_COUNT
        self.root: str = PIH.NAME
        self.checker: Callable[[str], bool] | None = checker
        self.service_description: ServiceDescription = SD
        self.allow_send_to_next_service_in_chain: dict[str, bool] = defaultdict(bool)

    def start(self, as_standalone: bool = False) -> bool:
        A.SE.add_isolated_arg()
        A.SE.add_arg(ADMIN_ALIAS, nargs="?", const="True", type=str, default="False")
        A.SE.add_arg(COUNT_ALIAS, nargs="?", const=1, type=int, default=DEFAULT_COUNT)
        service_desctiption: ServiceDescription | None = (
            A.SRV_A.create_support_service_or_master_service_description(
                self.service_description
            )
        )
        if A.SRV.is_service_as_support(service_desctiption):
            MobileHelperService.is_admin = lw(A.SE.named_arg(ADMIN_ALIAS)) in [
                "1",
                "true",
                "yes",
            ]
        else:
            MobileHelperService.is_admin = False
        if ne(service_desctiption):
            A.SRV_A.serve(
                service_desctiption,
                self.service_call_handler,
                MobileHelperService.service_starts_handler,
                as_standalone=as_standalone,
                isolate=ISOLATED,
            )
            return True
        return False

    def create_mobile_helper(
        self,
        sender: str,
        external_flags: int | None = None,
        recipient: str | None = None,
    ) -> Api:
        is_cli: bool = sender_is_cli(sender)
        if is_cli:
            external_flags = BM.add(external_flags, Flags.CLI)
        stdin: Stdin = Stdin()
        session: MobileSession = MobileSession(sender, external_flags)
        output: MobileOutput = MobileOutput(session)
        try:
            session.say_hello(recipient)
            output.write_line(
                j(
                    (
                        (
                            j(
                                (
                                    "Добро пожаловать, ",
                                    nl(
                                        j(
                                            (
                                                output.user.get_formatted_given_name(
                                                    session.user_given_name
                                                ),
                                                "!",
                                            )
                                        )
                                    ),
                                )
                            )
                            if not BM.has(session.flags, Flags.ONLY_RESULT)
                            else None
                        ),
                        " ",
                        A.CT_V.WAIT,
                        " ",
                        i("Ожидайте..."),
                    )
                )
            )
        except NotFound as error:
            output.error(
                "К сожалению, не могу идентифицировать Вас. ИТ отдел добавит Вас после окончания процедуры идентификации."
            )
            raise error
        as_admin: bool = is_cli
        if not as_admin:
            try:
                as_admin = A.C_U.by_group(
                    A.R_U.by_telephone_number(sender).data, A.CT_AD.Groups.Admin
                )
            except NotFound as _:
                pass
        input: MobileInput = MobileInput(
            stdin,
            MobileUserInput(),
            MobileMarkInput(),
            output,
            session,
            [None, -1][as_admin],
        )
        api: Api = Api(PIH(input, output, session), stdin)
        if is_cli:
            api.external_flags = external_flags
        return api

    @staticmethod
    def say_good_bye(mobile_helper: Api) -> None:
        mobile_helper.say_good_bye()
        mobile_helper.show_good_bye = False

    def pih_handler(
        self,
        sender: str,
        line: str | None = None,
        sender_user: User | None = None,
        external_flags: int | None = None,
        chat_id: str | None = None,
        return_result_key: str | None = None,
        args: tuple[Any] | None = None,
    ) -> None:
        mobile_helper: Api | None = None
        is_cli: bool = sender_is_cli(sender)
        while True:
            try:
                if MobileHelperService.is_client_new(sender):
                    A.IW.remove(A.CT_P.NAMES.PERSON_PIN, sender)
                    if is_cli or Api.check_for_starts_with_pih_keyword(line):
                        self.allow_send_to_next_service_in_chain[sender] = (
                            self.is_client_stack_full()
                        )
                        if not self.allow_send_to_next_service_in_chain[sender]:
                            MobileHelperService.mobile_helper_client_map[sender] = (
                                self.create_mobile_helper(
                                    sender, external_flags, chat_id
                                )
                            )
                    else:
                        self.allow_send_to_next_service_in_chain[sender] = False
                else:
                    self.allow_send_to_next_service_in_chain[sender] = False
                if sender in MobileHelperService.mobile_helper_client_map:
                    mobile_helper = MobileHelperService.mobile_helper_client_map[sender]
                    if is_cli and not mobile_helper.wait_for_input():
                        if not Api.check_for_starts_with_pih_keyword(line):
                            line = js((self.root, line))
                    if mobile_helper.do_pih(
                        line, sender_user, external_flags, return_result_key, args
                    ):
                        if mobile_helper.show_good_bye:
                            if not mobile_helper.is_only_result:
                                MobileHelperService.say_good_bye(mobile_helper)
                break
            except NotFound:
                break
            except InternalInterrupt as interruption:
                if interruption.type == InterruptionTypes.NEW_COMMAND:
                    line = mobile_helper.line
                    if not Api.check_for_starts_with_pih_keyword(line):
                        MobileHelperService.say_good_bye(mobile_helper)
                        break
                elif interruption.type in (
                    InterruptionTypes.TIMEOUT,
                    InterruptionTypes.EXIT,
                ):
                    MobileHelperService.say_good_bye(mobile_helper)
                    break

    def is_client_stack_full(self) -> bool:
        max_client_count: int | None = self.max_client_count
        if n(max_client_count):
            max_client_count = MobileHelperService.count()
        return len(MobileHelperService.mobile_helper_client_map) == max_client_count

    @staticmethod
    def is_client_new(value: str) -> bool:
        return value not in MobileHelperService.mobile_helper_client_map

    def receive_message_handler(
        self,
        message_text: str,
        sender: str,
        external_flags: int | None = None,
        chat_id: str | None = None,
        return_result_key: str | None = None,
        args: tuple[Any] | None = None,
    ) -> None:
        interruption: AddressedInterruption | None = None
        while True:
            try:
                if e(interruption):
                    self.pih_handler(
                        sender,
                        message_text,
                        None,
                        external_flags,
                        chat_id,
                        return_result_key,
                        args,
                    )
                else:
                    for recipient_user in interruption.recipient_user_list():
                        recipient_user: User = recipient_user
                        self.pih_handler(
                            recipient_user.telephoneNumber,
                            js((self.root, interruption.command_name)),
                            interruption.sender_user,
                            interruption.flags,
                        )
                    interruption = None
                break
            except AddressedInterruption as local_interruption:
                interruption = local_interruption

    def receive_message_handler_thread_handler(self, message: WhatsAppMessage) -> None:
        self.receive_message_handler(
            message.message,
            message.sender,
            message.flags,
            message.chatId,
            message.return_result_key,
            message.args,
        )

    def service_call_handler(
        self,
        sc: SC,
        parameter_list: ParameterList,
        subscribtion_result: SubscribtionResult | None,
    ) -> Any:
        if sc == A.CT_SC.send_event:
            if nn(subscribtion_result) and subscribtion_result.result:
                if subscribtion_result.type == A.CT_SubT.ON_RESULT_SEQUENTIALLY:
                    message: WhatsAppMessage | None = A.D_Ex_E.whatsapp_message(
                        parameter_list
                    )

                    if nn(message):
                        if (
                            A.D.get_by_value(A.CT_ME_WH_W.Profiles, message.profile_id)
                            == A.CT_ME_WH_W.Profiles.IT
                        ):
                            allow_for_group: bool = ne(
                                message.chatId
                            ) and sender_is_cli(message.chatId)
                            if n(message.chatId) or allow_for_group:
                                telephone_number: str = if_else(
                                    allow_for_group, message.chatId, message.sender
                                )
                                if allow_for_group:
                                    message.chatId = message.sender
                                    message.sender = telephone_number
                                if n(self.checker) or self.checker(telephone_number):
                                    if self.is_client_stack_full():
                                        return True
                                    else:
                                        if (
                                            telephone_number
                                            in self.allow_send_to_next_service_in_chain
                                        ):
                                            del self.allow_send_to_next_service_in_chain[
                                                telephone_number
                                            ]
                                        PIHThread(
                                            self.receive_message_handler_thread_handler,
                                            args=[message],
                                        )
                                        while (
                                            telephone_number
                                            not in self.allow_send_to_next_service_in_chain
                                        ):
                                            pass
                                        return self.allow_send_to_next_service_in_chain[
                                            telephone_number
                                        ]
                                else:
                                    if (
                                        telephone_number
                                        in MobileHelperService.mobile_helper_client_map
                                    ):
                                        del MobileHelperService.mobile_helper_client_map[
                                            telephone_number
                                        ]
                                    return True
            return False
        return None

    @staticmethod
    def service_starts_handler() -> None:
        A.O.write_line(nl())
        A.O.blue("Configuration:")
        as_admin: bool = MobileHelperService.as_admin()
        with A.O.make_indent(1):
            A.O.value("As admin", str(as_admin))
            if not as_admin:
                A.O.value("Count", str(MobileHelperService.count()))
        A.SRV_A.subscribe_on(
            A.CT_SC.send_event,
            A.CT_SubT.ON_RESULT_SEQUENTIALLY,
            SD.name,
        )
        profile = A.CT_ME_WH_W.Profiles
        if as_admin:
            space: str = "     "

            MIO.create_output(A.D.get(A.CT_ME_WH.GROUP.PIH_CLI)).write_line(
                j(
                    (
                        " ",
                        A.CT_V.ROBOT,
                        " ",
                        nl(b("Pih cli запущен...")),
                        nl(js((space, b("Сервер:"), A.SYS.host()))),
                        js((space, b("Версия:"), VERSION_STRING)),
                        nl(),
                        space,
                        LINE,
                        nl(),
                        get_wappi_status(space, profile.IT),
                        nl() * 2,
                        get_wappi_status(space, profile.CALL_CENTRE),
                        nl() * 2,
                        get_wappi_status(space, profile.MARKETER),
                    )
                )
            )

        for item in one(A.R_F.find("@mobile_helper_import_module_list")).text.splitlines():  # type: ignore
            A.R_F.execute(item)



from enum import Enum, auto

import ipih

from pih import A
from pih.tools import j, js, b
from pih.consts.hosts import Hosts
from pih.consts import SessionFlags, SPLITTER
from pih.collections.service import ServiceDescription
from pih.collections import StorageVariableHolder, IntStorageVariableHolder


NAME: str = "MobileHelper"

DEFAULT_COUNT = 100
ADMIN_ALIAS: str = "admin"
COUNT_ALIAS: str = "count"
VERSION: str = "0.19"

HOST = Hosts.WS255

SD: ServiceDescription = ServiceDescription(
    name=NAME,
    description="Mobile helper service",
    host=HOST.NAME,
    standalone_name=A.CT_SR.MOBILE_HELPER.standalone_name,
    use_standalone=True,
    version=VERSION,
)

MOBILE_HELPER_USER_SETTINGS_NAME: str = "mobile_helper_user_settings"

USER_DATA_INPUT_TIMEOUT: int = 180

MOBILE_HELPER_USER_DATA_INPUT_TIMEOUT: StorageVariableHolder = IntStorageVariableHolder(
    "MOBILE_HELPER_USER_DATA_INPUT_TIMEOUT",
    USER_DATA_INPUT_TIMEOUT,
)


class InterruptionTypes:
    NEW_COMMAND = 1
    TIMEOUT = 2
    EXIT = 3
    BACK = 4
    CANCEL = 5


class INPUT_TYPE:
    INDEX: int = 1


FROM_FILE_REDIRECT_SYMBOL: str = "<"
FILE_PATTERN_LIST: tuple[str, ...] = (
    j(("file", SPLITTER)),
    FROM_FILE_REDIRECT_SYMBOL,
    "file=",
)  # type: ignore
PARAMETER_PATTERN: str = r"\B(\$[\w+а-яА-Я]+)(:[\w \(\)\.\,а-яА-Я]+)?"

Groups = A.CT_AD.Groups

MAX_MESSAGE_LINE_LENGTH = 12


class MessageType(Enum):
    SEPARATE_ONCE = auto()
    SEPARATED = auto()


class COMMAND_KEYWORDS:
    PIH: tuple[str, ...] = (A.root.NAME, A.root.NAME_ALT)  # type: ignore
    EXIT: list[str] = ["exit", "выход"]
    BACK: list[str] = ["back", "назад"]
    FIND: list[str] = ["find", "поиск", "search", "найти"]
    CREATE: list[str] = ["create", "создать", "+"]
    CHECK: list[str] = ["check", "проверить"]
    ADD: list[str] = ["add", "добавить", "+"]
    PASSWORD: list[str] = ["password", "пароль"]
    USER: list[str] = ["user^s", "пользовател^ь"]
    POLIBASE: list[str] = ["polibase", "полибейс"]
    NOTES: list[str] = ["note^s", "заметк^и"]
    JOURNALS: list[str] = ["journal^s", "журнал^ы"]
    RUN: list[str] = ["run", "запуск"]
    PATIENT: list[str] = ["patient", "пациент"]
    DOCTOR: list[str] = ["doc^tor", "доктор", "врач"]
    JOKE: list[str] = ["joke", "шутка", "анекдот"]
    ASK: list[str] = ["ask"]
    ASK_YES_NO: list[str] = ["ask_yes_no"]
    SCAN: list[str] = ["scan^ed"]
    REGISTRY: list[str] = ["registry", "реестр"]
    CARD: list[str] = ["card", "карт"]


YES_VARIANTS: tuple[str, ...] = ("1", "yes", "ok", "да")
YES_LABEL: str = js(("", A.CT.VISUAL.BULLET, b("Да"), "- отправьте", A.O.get_number(1)))
NO_LABEL: str = js(("", A.CT.VISUAL.BULLET, b("Нет"), "- отправьте", A.O.get_number(0)))


class Flags(Enum):
    CYCLIC = 1
    ADDRESS = 2
    ALL = 4
    ADDRESS_AS_LINK = 8
    FORCED = 16
    SILENCE = 32
    HELP = 64
    SILENCE_NO = 128
    SILENCE_YES = 256
    CLI = SessionFlags.CLI.value
    ONLY_RESULT = 1024
    TEST = 2048
    SETTINGS = 4096


class FLAG_KEYWORDS:
    ALL_SYMBOL: str = "*"
    ADDRESS_SYMBOL: str = ">"
    LINK_SYMBOL: str = ADDRESS_SYMBOL * 2
    SILENCE: str = "_"
    ONLY_RESULT: str = SILENCE * 2


COMMAND_LINK_SYMBOL: str = "@"

FLAG_MAP: dict[str, Flags] = {
    "-0": Flags.CYCLIC,
    "-t": Flags.TEST,
    "to": Flags.ADDRESS,
    FLAG_KEYWORDS.ADDRESS_SYMBOL: Flags.ADDRESS,
    "!": Flags.FORCED,
    FLAG_KEYWORDS.SILENCE: Flags.SILENCE,
    "_0": Flags.SILENCE_NO,
    "_1": Flags.SILENCE_YES,
    "-n": Flags.SILENCE_NO,
    "-y": Flags.SILENCE_YES,
    FLAG_KEYWORDS.ALL_SYMBOL: Flags.ALL,
    FLAG_KEYWORDS.ONLY_RESULT: Flags.ONLY_RESULT,
    "all": Flags.ALL,
    "все": Flags.ALL,
    "всё": Flags.ALL,
    "link": Flags.ADDRESS_AS_LINK,
    FLAG_KEYWORDS.LINK_SYMBOL: Flags.ADDRESS_AS_LINK,
    "?": Flags.HELP,
    "-s": Flags.SETTINGS,
}

ADMIN_GROUPS: list[Groups] = [Groups.Admin]

COMMAND_NAME_BASE_DELEMITER: str = "^"

WORKSTATION_CHECK_WORD: str = "Workstation"

VERSION_STRING: str = js((VERSION, "⚙️"))
MODULE_NAME: str = j((A.root.NAME, "mio"), "-")
